#!/usr/bin/env python
#
# Copyright (c) 2012 Cloudera, Inc. All rights reserved.
#
 
'''
This script is provided by Cloudera Manager to initiate a fencing
request to a namenode.  It attempts to ask a remote agent to shut
down the other namenode, initially through the supervisor,
and, if that doesn't work, using kill, based on the ports
the NN is listening to.

Inputs:
 - target_namenodeid is specified as environment variables
 - the environment variables include the HDFS configuration, with dashes converted to underscores
Output:
 - Exits with status 0 on a succesful failure, and
   otherwise exists with a non-zero error code.
 - Logging and debugging information is sent to stderr
   and stdout.
'''

import socket
import urllib
import urllib2
import os
import sys

# exit codes
COULD_NOT_FIND_NAMENODE_ID = 3
COULD_NOT_FIND_AGENT_URL = 4
REMOTE_RETURNED_FENCE_FAILED = 2
TARGET_NOT_SPECIFIED = 5

namenode_id = os.environ.get("target_namenodeid")
if namenode_id is None:
  print >>sys.stderr, "Target namenode id id not specified."
  sys.exit(TARGET_NOT_SPECIFIED)

agent_url = os.environ.get("dfs_ha_fencing_cloudera_manager_agent_url_" + namenode_id)
secret_key = file("cloudera_manager_agent_fencer_secret_key.txt").read().rstrip()

if agent_url is None:
  print >>sys.stderr, "Could not find agent url for %s" % (namenode_id,)
  sys.exit(COULD_NOT_FIND_AGENT_URL)

print "Calling: ", agent_url

timeout_millis = os.environ.get("dfs_ha_fencing_cloudera_manager_timeout_millis")
timeout_millis = int(timeout_millis)
timeout_seconds = timeout_millis / 1000.0

# Set timeout on request.
print "Using socket timeout: %s", (timeout_seconds,)
socket.setdefaulttimeout(timeout_seconds)
try:
  result = urllib2.urlopen(agent_url, data=urllib.urlencode(dict(secret_key=secret_key)))
except urllib2.HTTPError, e:
  # HTTP Errors can be read() like results, and have codes too.
  result = e

data = result.read()
print data

if result.code == 200:
  # This check is redundant with the 200 code check.
  if data.splitlines()[0] == "OK":
  	print "Fence successful"
  	sys.exit(0)

print "Fence unsuccessful."
sys.exit(REMOTE_RETURNED_FENCE_FAILED)
